Package: dmrseq
Type: Package
Title: Detection and inference of differentially methylated regions
        from Whole Genome Bisulfite Sequencing
Version: 1.32.0
Authors@R: c(person("Keegan", "Korthauer", role = c("cre", "aut"), 
                    email = "keegan@stat.ubc.ca", 
                    comment = c(ORCID = "0000-0002-4565-1654")),
             person("Rafael", "Irizarry", role = "aut",
                    email = "rafa@jimmy.harvard.edu",
                    comment = c(ORCID = "0000-0002-3944-4309")),
             person("Yuval", "Benjamini", role = "aut"),
             person("Sutirtha", "Chakraborty", role = "aut"))
Description: This package implements an approach for scanning the
  genome to detect and perform accurate inference on differentially 
  methylated regions from Whole Genome Bisulfite Sequencing data. 
  The method is based on comparing detected regions 
  to a pooled null distribution, that can be implemented even when as 
  few as two samples per population are available. 
  Region-level statistics are obtained by fitting a generalized 
  least squares (GLS) regression model with a nested autoregressive  
  correlated error structure for the effect of interest on transformed  
  methylation proportions. 
License: MIT + file LICENSE
Depends: R (>= 3.5), bsseq
Imports: GenomicRanges, nlme, ggplot2, S4Vectors, RColorBrewer,
        bumphunter, DelayedMatrixStats (>= 1.1.13), matrixStats,
        BiocParallel, outliers, methods, locfit, IRanges, grDevices,
        graphics, stats, utils, annotatr, AnnotationHub, rtracklayer,
        Seqinfo, splines
Suggests: knitr, rmarkdown, BiocStyle,
        TxDb.Hsapiens.UCSC.hg19.knownGene, org.Hs.eg.db
biocViews: ImmunoOncology, DNAMethylation, Epigenetics,
        MultipleComparison, Software, Sequencing,
        DifferentialMethylation, WholeGenome, Regression,
        FunctionalGenomics
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/dmrseq
git_branch: RELEASE_3_23
git_last_commit: c4da253
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:01:04 UTC; biocbuild
Author: Keegan Korthauer [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-4565-1654>),
  Rafael Irizarry [aut] (ORCID: <https://orcid.org/0000-0002-3944-4309>),
  Yuval Benjamini [aut],
  Sutirtha Chakraborty [aut]
Maintainer: Keegan Korthauer <keegan@stat.ubc.ca>
Built: R 4.6.0; ; 2026-04-29 04:40:52 UTC; unix
