# Generated by roxygen2: do not edit by hand

export(dmrseq)
export(getAnnot)
export(meanDiff)
export(plotDMRs)
export(plotEmpiricalDistribution)
export(simDMRs)
import(GenomicRanges)
import(S4Vectors)
import(annotatr)
import(bsseq)
import(ggplot2)
import(nlme)
importClassesFrom(bsseq,BSseq)
importFrom(AnnotationHub,AnnotationHub)
importFrom(AnnotationHub,query)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(BiocParallel,register)
importFrom(DelayedMatrixStats,colMedians)
importFrom(DelayedMatrixStats,rowDiffs)
importFrom(DelayedMatrixStats,rowMads)
importFrom(DelayedMatrixStats,rowMeans2)
importFrom(DelayedMatrixStats,rowSums2)
importFrom(IRanges,IRanges)
importFrom(RColorBrewer,brewer.pal)
importFrom(Seqinfo,genome)
importFrom(bumphunter,clusterMaker)
importFrom(bumphunter,getSegments)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,hcl)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,text)
importFrom(locfit,locfit)
importFrom(locfit,lp)
importFrom(matrixStats,rowRanges)
importFrom(methods,is)
importFrom(outliers,grubbs.test)
importFrom(rtracklayer,liftOver)
importFrom(splines,ns)
importFrom(stats,anova)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,preplot)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,runif)
importFrom(utils,combn)
importMethodsFrom(bsseq,pData)
importMethodsFrom(bsseq,sampleNames)
importMethodsFrom(bsseq,seqnames)
importMethodsFrom(bsseq,start)
importMethodsFrom(bsseq,width)
